@@ -5,7 +5,7 @@ use warnings;
 use Moo;
 use TheParameterizedRole;
 
-TheParameterizedRole->apply(
+TheParameterizedRole->apply_roles_to_target(
     [   { attribute => 'foo', method => 'xxx' },
         { attribute => 'bar', method => 'yyy' }
     ]
